<?php
/**
 * @copyright	Copyright (C) 2005 - 2013 Open Source Matters, Inc. All rights reserved.
 * @license		GNU General Public License version 2 or later; see LICENSE.txt
 */

defined('JPATH_BASE') or die;

use Joomla\CMS\HTML\HTMLHelper;

/**
 * Supports a modal article picker.
 *
 * @package		Joomla.Administrator
 * @subpackage	com_content
 * @since		1.6
 */
class JFormFieldModal_DJAlbum extends JFormField
{
	/**
	 * The form field type.
	 *
	 * @var		string
	 * @since	1.6
	 */
	protected $type = 'Modal_DJAlbum';

	/**
	 * Method to get the field input markup.
	 *
	 * @return	string	The field input markup.
	 * @since	1.6
	 */
	protected function getInput()
	{
		$lang = JFactory::getLanguage();
		$lang->load('com_djmediatools', JPATH_ADMINISTRATOR, 'en-GB', false, false);
		$lang->load('com_djmediatools', JPATH_ADMINISTRATOR . '/components/com_djmediatools', 'en-GB', false, false);
		$lang->load('com_djmediatools', JPATH_ADMINISTRATOR, null, true, false);
		$lang->load('com_djmediatools', JPATH_ADMINISTRATOR . '/components/com_djmediatools', null, true, false);
		
		if(version_compare(JVERSION, '4', '>=')) {
			
			// The active album id field.
			$value = (int) $this->value > 0 ? (int) $this->value : '';
			
			// Create the modal id.
			$modalId = 'DJAlbum_' . $this->id;
			
			// Add the modal field script to the document head.
			HTMLHelper::_('script', 'system/fields/modal-fields.min.js', array('version' => 'auto', 'relative' => true));
			
			// Script to proxy the select modal function to the modal-fields.js file.
			static $scriptSelect = null;
			
			if (is_null($scriptSelect))
			{
				$scriptSelect = array();
			}
			
			if (!isset($scriptSelect[$this->id]))
			{
				JFactory::getDocument()->addScriptDeclaration("
					function jSelectAlbum_" . $this->id . "(id, img, title) {
						window.processModalSelect('DJAlbum', '" . $this->id . "', id, title);
					}
				");
				$scriptSelect[$this->id] = true;
			}
			
			$urlSelect = 'index.php?option=com_djmediatools&amp;view=categories&amp;layout=modal&amp;tmpl=component&amp;f_name=jSelectAlbum_'.$this->id;
			$modalTitle = JText::_('COM_DJMEDIATOOLS_SELECT_AN_ALBUM');
			
			if ($value)
			{
				$db    = JFactory::getDbo();
				$query = $db->getQuery(true)
				->select($db->quoteName('title'))
				->from($db->quoteName('#__djmt_albums'))
				->where($db->quoteName('id') . ' = ' . (int) $value);
				$db->setQuery($query);
			
				try
				{
					$title = $db->loadResult();
				}
				catch (\RuntimeException $e)
				{
					JFactory::getApplication()->enqueueMessage($e->getMessage(), 'error');
				}
			}
			
			$title = empty($title) ? JText::_('COM_DJMEDIATOOLS_SELECT_AN_ALBUM') : htmlspecialchars($title, ENT_QUOTES, 'UTF-8');
			
			$html  = '';
			$html .= '<span class="input-group">';
			$html .= '	<input class="form-control" id="' . $this->id . '_name" type="text" value="' . $title . '" disabled="disabled" size="35">';
			$html .= '	<span class="input-group-append">';
			$html .= '		<a class="btn btn-primary hasTooltip' . ($value ? ' sr-only' : '') . '"'
							. ' id="' . $this->id . '_select"'
							. ' data-toggle="modal"'
							. ' role="button"'
							. ' href="#ModalSelect' . $modalId . '"'
							. ' title="' . HTMLHelper::tooltipText('COM_DJMEDIATOOLS_SELECT_AN_ALBUM') . '">'
							. '<span class="icon-pictures" aria-hidden="true"></span> ' . JText::_('JSELECT')
						. '</a>';
			
			$html .= '		<a'
							. ' class="btn btn-secondary' . ($value ? '' : ' sr-only') . '"'
							. ' id="' . $this->id . '_clear"'
							. ' href="#"'
							. ' onclick="window.processModalParent(\'' . $this->id . '\'); return false;">'
							. '<span class="icon-remove" aria-hidden="true"></span>' . JText::_('JCLEAR')
						. '</a>';
			$html .= '</span></span>';
			
			$html .= HTMLHelper::_(
					'bootstrap.renderModal',
					'ModalSelect' . $modalId,
					array(
							'title'       => $modalTitle,
							'url'         => $urlSelect,
							'height'      => '400px',
							'width'       => '800px',
							'bodyHeight'  => 70,
							'modalWidth'  => 80,
							'footer'      => '<a role="button" class="btn btn-secondary" data-dismiss="modal" aria-hidden="true">'
							. JText::_('JLIB_HTML_BEHAVIOR_CLOSE') . '</a>',
					)
			);
			
			// Note: class='required' for client side validation.
			$class = $this->required ? ' class="required modal-value"' : '';
			
			$html .= '<input type="hidden" id="' . $this->id . '_id" ' . $class . ' data-required="' . (int) $this->required . '" name="' . $this->name
			. '" data-text="' . htmlspecialchars(JText::_('COM_DJMEDIATOOLS_SELECT_AN_ALBUM', true), ENT_COMPAT, 'UTF-8') . '" value="' . $value . '">';
			
			return $html;
		
		} else {
			
			// Load the modal behavior script.
			JHtml::_('behavior.modal');
			
			// Build the script.
			$script = array();
			$script[] = '	function jSelectAlbum_'.$this->id.'(catid, img, title) {';
			$script[] = '		document.getElementById("'.$this->id.'_id").value = catid;';
			$script[] = '		document.getElementById("'.$this->id.'_name").value = title;';
			$script[] = '		SqueezeBox.close();';
			$script[] = '	}';
	
			// Add the script to the document head.
			JFactory::getDocument()->addScriptDeclaration(implode("\n", $script));
	
			// Setup variables for display.
			$html	= array();
			$link = 'index.php?option=com_djmediatools&amp;view=categories&amp;layout=modal&amp;tmpl=component&amp;f_name=jSelectAlbum_'.$this->id;
	
			$db	= JFactory::getDBO();
			$db->setQuery(
				'SELECT title' .
				' FROM #__djmt_albums' .
				' WHERE id = '.(int) $this->value
			);
			$title = $db->loadResult();
			
			if ($error = $db->getErrorMsg()) {
				JError::raiseWarning(500, $error);
			}
	
			if (empty($title)) {
				$title = JText::_('COM_DJMEDIATOOLS_SELECT_AN_ALBUM');
			}
			$title = htmlspecialchars($title, ENT_QUOTES, 'UTF-8');
			$options = '{handler: \'iframe\', size: {x: \'100%\', y: \'100%\'}, onOpen: function() { window.addEvent(\'resize\', function(){ this.resize({x: window.getSize().x - 100, y: window.getSize().y - 100}, true); }.bind(this) ); window.fireEvent(\'resize\'); }}';
			
			$version = new JVersion;
			if (version_compare($version->getShortVersion(), '3.0.0', '<')) {
				// The current user display field.
				$html[] = '<div class="fltlft">';
				$html[] = '  <input type="text" id="'.$this->id.'_name" value="'.$title.'" disabled="disabled" size="35" />';
				$html[] = '</div>';
		
				// The user select button.
				$html[] = '<div class="button2-left">';
				$html[] = '  <div class="blank">';
				$html[] = '	<a class="modal" title="'.JText::_('COM_DJMEDIATOOLS_SELECT_AN_ALBUM').'"  href="'.$link.'&amp;'.JSession::getFormToken().'=1" rel="'.$options.'">'.JText::_('COM_DJMEDIATOOLS_SELECT').'</a>';
				$html[] = '  </div>';
				$html[] = '</div>';
				
			} else {
				$html[] = '<span class="input-append">';
				// The current user display field.
				$html[] = '<input class="input-medium" type="text" id="'.$this->id.'_name" value="'.$title.'" disabled="disabled" size="35" />';
				
				// The user select button.
				$html[] = '<a class="modal btn" title="'.JText::_('COM_DJMEDIATOOLS_SELECT_AN_ALBUM').'"  href="'.$link.'&amp;'.JSession::getFormToken().'=1" rel="'.$options.'">'.JText::_('COM_DJMEDIATOOLS_SELECT').'</a>';
				$html[] = '<a class="btn" title="'.JText::_('COM_DJMEDIATOOLS_CLEAR').'"  href="#" onclick="jSelectAlbum_'.$this->id.'(\'\', \'\', \'\'); return false;">'.JText::_('COM_DJMEDIATOOLS_CLEAR').'</a>';
				$html[] = '</span>';
			}
			
			// The active article id field.
			if (0 == (int)$this->value) {
				$value = '';
			} else {
				$value = (int)$this->value;
			}
	
			// class='required' for client side validation
			$class = '';
			if ($this->required) {
				$class = ' class="required modal-value"';
			}
	
			$html[] = '<input type="hidden" id="'.$this->id.'_id"'.$class.' name="'.$this->name.'" value="'.$value.'" />';
	
			return implode("\n", $html);
		}
	}
}
